package org.jvnet.annox.model;

import org.jvnet.annox.model.XAnnotationField.XAnnotation;
import org.jvnet.annox.model.XAnnotationField.XAnnotationArray;
import org.jvnet.annox.model.XAnnotationField.XBoolean;
import org.jvnet.annox.model.XAnnotationField.XBooleanArray;
import org.jvnet.annox.model.XAnnotationField.XByte;
import org.jvnet.annox.model.XAnnotationField.XByteArray;
import org.jvnet.annox.model.XAnnotationField.XChar;
import org.jvnet.annox.model.XAnnotationField.XCharArray;
import org.jvnet.annox.model.XAnnotationField.XClass;
import org.jvnet.annox.model.XAnnotationField.XClassArray;
import org.jvnet.annox.model.XAnnotationField.XDouble;
import org.jvnet.annox.model.XAnnotationField.XDoubleArray;
import org.jvnet.annox.model.XAnnotationField.XEnum;
import org.jvnet.annox.model.XAnnotationField.XEnumArray;
import org.jvnet.annox.model.XAnnotationField.XFloat;
import org.jvnet.annox.model.XAnnotationField.XFloatArray;
import org.jvnet.annox.model.XAnnotationField.XInt;
import org.jvnet.annox.model.XAnnotationField.XIntArray;
import org.jvnet.annox.model.XAnnotationField.XLong;
import org.jvnet.annox.model.XAnnotationField.XLongArray;
import org.jvnet.annox.model.XAnnotationField.XShort;
import org.jvnet.annox.model.XAnnotationField.XShortArray;
import org.jvnet.annox.model.XAnnotationField.XString;
import org.jvnet.annox.model.XAnnotationField.XStringArray;

/**
 * Annotation visitor.
 * 
 * @param <T>
 *            Visitor return type.
 * @author Aleksei Valikov
 */
public interface XAnnotationVisitor<T> {

	/**
	 * Visits an annotation.
	 * 
	 * @param annotation
	 *            annotation to be visited.
	 * @return Result of the visit.
	 */
	public T visitAnnotation(org.jvnet.annox.model.XAnnotation annotation);

	/**
	 * Visits a boolean field.
	 * @param field field to be visited.
	 * @return Result of the visit.
	 */
	public T visitBooleanField(XBoolean field);

	/**
	 * Visits a byte field.
	 * @param field field to be visited.
	 * @return Result of the visit.
	 */
	public T visitByteField(XByte field);

	/**
	 * Visits a int field.
	 * @param field field to be visited.
	 * @return Result of the visit.
	 */
	public T visitIntField(XInt field);

	/**
	 * Visits a long field.
	 * @param field field to be visited.
	 * @return Result of the visit.
	 */
	public T visitLongField(XLong field);

	/**
	 * Visits a short field.
	 * @param field field to be visited.
	 * @return Result of the visit.
	 */
	public T visitShortField(XShort field);

	/**
	 * Visits a char field.
	 * @param field field to be visited.
	 * @return Result of the visit.
	 */
	public T visitCharField(XChar field);

	/**
	 * Visits a class field.
	 * @param field field to be visited.
	 * @return Result of the visit.
	 */
	public T visitClassField(XClass field);

	/**
	 * Visits a double field.
	 * @param field field to be visited.
	 * @return Result of the visit.
	 */
	public T visitDoubleField(XDouble field);

	/**
	 * Visits a float field.
	 * @param field field to be visited.
	 * @return Result of the visit.
	 */
	public T visitFloatField(XFloat field);

	/**
	 * Visits a enum field.
	 * @param field field to be visited.
	 * @return Result of the visit.
	 */
	public T visitEnumField(XEnum field);

	/**
	 * Visits a string field.
	 * @param field field to be visited.
	 * @return Result of the visit.
	 */
	public T visitStringField(XString field);

	/**
	 * Visits a annotation field.
	 * @param field field to be visited.
	 * @return Result of the visit.
	 */
	public T visitAnnotationField(XAnnotation field);

	/**
	 * Visits a boolean array field.
	 * @param field field to be visited.
	 * @return Result of the visit.
	 */
	public T visitBooleanArrayField(XBooleanArray field);

	/**
	 * Visits a byte array field.
	 * @param field field to be visited.
	 * @return Result of the visit.
	 */
	public T visitByteArrayField(XByteArray field);

	/**
	 * Visits a int array field.
	 * @param field field to be visited.
	 * @return Result of the visit.
	 */
	public T visitIntArrayField(XIntArray field);

	/**
	 * Visits a long array field.
	 * @param field field to be visited.
	 * @return Result of the visit.
	 */
	public T visitLongArrayField(XLongArray field);

	/**
	 * Visits a short array field.
	 * @param field field to be visited.
	 * @return Result of the visit.
	 */
	public T visitShortArrayField(XShortArray field);

	/**
	 * Visits a char array field.
	 * @param field field to be visited.
	 * @return Result of the visit.
	 */
	public T visitCharArrayField(XCharArray field);

	/**
	 * Visits a class array field.
	 * @param field field to be visited.
	 * @return Result of the visit.
	 */
	public T visitClassArrayField(XClassArray field);

	/**
	 * Visits a double array field.
	 * @param field field to be visited.
	 * @return Result of the visit.
	 */
	public T visitDoubleArrayField(XDoubleArray field);

	/**
	 * Visits a float array field.
	 * @param field field to be visited.
	 * @return Result of the visit.
	 */
	public T visitFloatArrayField(XFloatArray field);

	/**
	 * Visits a enum array field.
	 * @param field field to be visited.
	 * @return Result of the visit.
	 */
	public T visitEnumArrayField(XEnumArray field);

	/**
	 * Visits a string array field.
	 * @param field field to be visited.
	 * @return Result of the visit.
	 */
	public T visitStringArrayField(XStringArray field);

	/**
	 * Visits an annotation array field.
	 * @param field field to be visited.
	 * @return Result of the visit.
	 */
	public T visitAnnotationArrayField(XAnnotationArray field);

}
