package org.jvnet.annox.parser.tests;

import java.io.IOException;
import java.io.InputStream;
import java.lang.annotation.Annotation;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.jvnet.annox.model.XAnnotation;
import org.jvnet.annox.model.XAnnotationField;
import org.jvnet.annox.parser.XAnnotationFieldParser;
import org.jvnet.annox.parser.XAnnotationParser;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class XAnnotationParserTest extends TestCase {

	public Element getElement(final String resourceName) throws Exception {
		InputStream is = null;
		try {
			is = getClass().getResourceAsStream(resourceName);
			final DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory
					.newInstance();
			documentBuilderFactory.setNamespaceAware(true);
			final DocumentBuilder documentBuilder = documentBuilderFactory
					.newDocumentBuilder();
			final Document document = documentBuilder.parse(is);
			return document.getDocumentElement();
		} finally {
			try {
				if (is != null) {
					is.close();
				}
			} catch (IOException ioe) {
				// ignore
			}
		}

	}

	public void testA() throws Exception {

		XAnnotationParser parser = new XAnnotationParser();

		final Element element = getElement("a.xml");

		final XAnnotation one = parser.parse(element);
		System.out.println(one.toString());
		final XAnnotation two = parser.parse(One.class.getAnnotation(A.class));
		System.out.println(two.toString());
		Assert.assertEquals("Annotations should be identical.", one, two);
		final Annotation oneResult = one.getResult();
		final Annotation twoResult = two.getResult();
		final XAnnotation three = parser.parse(oneResult);
		System.out.println(three.toString());
		final XAnnotation four = parser.parse(twoResult);
		System.out.println(four.toString());
		Assert.assertEquals("Annotations should be identical.", three, four);
	}

	public void testD() throws Exception {

		XAnnotationParser parser = XAnnotationParser.GENERIC;

		final Element element = getElement("d.xml");

		final XAnnotation one = parser.parse(element);
		System.out.println(one.toString());
		final XAnnotation two = parser
				.parse(Three.class.getAnnotation(D.class));
		System.out.println(two.toString());
		Assert.assertEquals("Annotations should be identical.", one, two);
		final Annotation oneResult = one.getResult();
		final Annotation twoResult = two.getResult();
		final XAnnotation three = parser.parse(oneResult);
		System.out.println(three.toString());
		final XAnnotation four = parser.parse(twoResult);
		System.out.println(four.toString());
		Assert.assertEquals("Annotations should be identical.", three, four);
	}

	public void testEquals() throws Exception {

		final Annotation one = One.class.getAnnotation(A.class);
		final Annotation two = Two.class.getAnnotation(A.class);
		System.out.println(one.toString());
		System.out.println(two.toString());
		Assert.assertEquals("Annotations should be identical.", one, two);
	}

	public void testF() throws Exception {

		final XAnnotationParser parser = XAnnotationParser.GENERIC;

		final XAnnotation one = parser.parse(Five.class.getAnnotation(F.class));

		final XAnnotation two = new XAnnotation(F.class,
				new XAnnotationField.XString("eman", "tset"));

		System.out.println(one.toString());
		System.out.println(two.toString());
		Assert.assertEquals("Annotations should be identical.", one, two);

	}

	public void testG() throws Exception {

		final XAnnotationParser parser = XAnnotationParser.GENERIC;

		final XAnnotation one = parser.parse(Six.class.getAnnotation(G.class));

		final XAnnotation two = new XAnnotation(G.class);

		System.out.println(one.toString());
		System.out.println(two.toString());
		Assert.assertEquals("Annotations should be identical.", one, two);

	}

	public void testField() throws Exception {

		final XAnnotationParser parser = XAnnotationParser.GENERIC;

		final Element element = getElement("field.xml");

		parser.parse(element);
		//		
		final XAnnotation one = parser.parse(Six.class.getAnnotation(G.class));

		final XAnnotation two = new XAnnotation(G.class);

		System.out.println(one.toString());
		System.out.println(two.toString());
		Assert.assertEquals("Annotations should be identical.", one, two);

	}

	public void testFieldBridge() throws Exception {

		final XAnnotationParser parser = XAnnotationParser.GENERIC;

		final Element element = getElement("fieldBridge.xml");

		XAnnotation annotation = parser.parse(element);

		final XAnnotation one = parser.parse(Six.class.getAnnotation(G.class));

		final XAnnotation two = new XAnnotation(G.class);

		System.out.println(one.toString());
		System.out.println(two.toString());
		Assert.assertEquals("Annotations should be identical.", one, two);

	}

	public void testH() throws Exception {

		final XAnnotationParser parser = XAnnotationParser.GENERIC;

		final Element element = getElement("h.xml");

		parser.parse(element);

		final XAnnotation one = parser.parse(Six.class.getAnnotation(G.class));

		final XAnnotation two = new XAnnotation(G.class);

		System.out.println(one.toString());
		System.out.println(two.toString());
		Assert.assertEquals("Annotations should be identical.", one, two);

	}

	public void testI() throws Exception {

		final XAnnotation d = new XAnnotation(D.class);

		final XAnnotation[] ds = new XAnnotation[] { d };
		XAnnotationFieldParser.GENERIC.construct("d", d, d.getClass());
		XAnnotationFieldParser.GENERIC.construct("d", ds, ds.getClass());
	}

	public void testJ() throws Exception {

		final XAnnotationParser parser = XAnnotationParser.GENERIC;

		final XAnnotation one = parser
				.parse(Seven.class.getAnnotation(J.class));

		final Element element = getElement("j.xml");

		final XAnnotation two = parser.parse(element);

		System.out.println(one.toString());
		System.out.println(two.toString());
		Assert.assertEquals("Annotations should be identical.", one, two);

	}

	public void testK() throws Exception {

		final XAnnotationParser parser = XAnnotationParser.GENERIC;

		final XAnnotation one = parser
				.parse(Eight.class.getAnnotation(K.class));

		final Element element = getElement("k.xml");

		final XAnnotation two = parser.parse(element);

		System.out.println(one.toString());
		System.out.println(two.toString());
		Assert.assertEquals("Annotations should be identical.", one, two);

	}

	public void testL() throws Exception {

		final XAnnotationParser parser = XAnnotationParser.GENERIC;

		final XAnnotation one = parser.parse(Nine.class.getAnnotation(L.class));

		final Element element = getElement("l.xml");

		final XAnnotation two = parser.parse(element);

		System.out.println(one.toString());
		System.out.println(two.toString());
		Assert.assertEquals("Annotations should be identical.", one, two);

	}

	public void testM() throws Exception {

		final XAnnotationParser parser = XAnnotationParser.GENERIC;

		final XAnnotation one = parser.parse(Ten.class.getAnnotation(M.class));

		final Element element = getElement("m.xml");

		final XAnnotation two = parser.parse(element);

		System.out.println(one.toString());
		System.out.println(two.toString());
		Assert.assertEquals("Annotations should be identical.", one, two);

	}
}
